(function() {
	function post(url, cb) {
		var xhr = new XMLHttpRequest();

		xhr.addEventListener("load", function() {
			var obj = JSON.parse(xhr.responseText);

			if (obj.error) {
				cb(obj.error);
			} else {
				cb(null, obj);
			}
		});

		xhr.addEventListener("error", function(err) {
			cb(err);
		});

		xhr.open("POST", url);
		xhr.send();
	}

	window.WebEvents = function() {
		var self = {};

		var cbs = {};
		var key = null;

		function emit(name, args) {
			if (!cbs[name])
				return;

			cbs[name].forEach(function(cb) {
				cb(args);
			});
		}

		function init(time) {
			post("/webevents/register", function(err, res) {

				// Retry on error
				if (err) {
					console.error(err);
					time = time || 1000;
					setTimeout(function() { init(time * 2) }, time);
					return;
				}

				key = res.key;
				await();
			});
		}

		function await(time) {
			post("/webevents/await/"+key, function(err, res) {

				// Retry registering on error
				if (err) {
					console.error(err);
					time = time || 1000;
					setTimeout(function() { init(time * 2) }, time);
					return;
				}

				res.forEach(function(evt) {
					emit(evt.name, evt.args);
				});

				await();
			});
		}

		self.on = function(name, cb) {
			if (!cbs[name])
				cbs[name] = [];

			cbs[name].push(cb);
		};

		init();

		return self;
	}
})();
